//
// osmtile.h
//
// Version 1.0
//
// Header for the Open Street Map (OSM) tile class.
//
// Copyright (c) 2008, Chris Hill.
// All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//
#include <iostream>

#include "osmtile.h"
#include <math.h>
#define PI 3.1415926535897932384626433832795


double DegToRad(double deg)
{
	return deg * PI / 180;
}

double RadToDeg(double rad)
{
	return rad * 180 / PI;
}

OSMTile::OSMTile()
{
	fxTile = -1;
	fyTile = -1;
	fZoom = -1;
}

OSMTile::~OSMTile()
{
	fxTile = -1;
}

int OSMTile::GetX()
{
	return fxTile;
}

int OSMTile::GetY()
{
	return fyTile;
}

int OSMTile::GetZoom()
{
	return fZoom;
}

void OSMTile::SetX(int value)
{
	fxTile = value;
	if (fyTile != -1 && fZoom != -1)
	{
		CalcLonLat();
	}
}

void OSMTile::SetY(int value)
{
	fyTile = value;
	if (fxTile != -1 && fZoom != -1)
	{
		CalcLonLat();
	}
}

void OSMTile::SetZoom(int value)
{
	fZoom = value;
	if (fxTile != -1 && fyTile != -1)
	{
		CalcLonLat();
	}
}

void OSMTile::CalcLonLat()
// calculate the edges of the current tile in long & lat
{
	double U;
	double V;
	double relY1;
	double relY2;
	double LimitY;
	double RangeY;
	
	U = 1 / (pow(2, fZoom));
	relY1 = fyTile * U;
	relY2 = relY1 + U;
	LimitY = PI;
	RangeY = 2 * LimitY;
	relY1 = LimitY - RangeY * relY1;
	relY2 = LimitY - RangeY * relY2;
	MaxLat = RadToDeg(atan(sinh(relY1)));
	MinLat = RadToDeg(atan(sinh(relY2)));
	V = 360 / (pow(2,fZoom));
	MinLon = -180 + fxTile * V;
	MaxLon = MinLon + V;
}

/*
double tile2long(int x, int z) 
{
	return x / pow(2.0, z) * 360.0 - 180;
}

double tile2lat(int y, int z) 
{
	double n = M_PI - 2.0 * M_PI * y / pow(2.0, z);
	return 180.0 / M_PI * atan(0.5 * (exp(n) - exp(-n)));
}
*/
